@@ -1,9 +1,9 @@
 package Net::DNS::RR::LOC;
 
 #
-# $Id: LOC.pm 1528 2017-01-18 21:44:58Z willem $
+# $Id: LOC.pm 1802 2020-08-31 13:44:20Z willem $
 #
-our $VERSION = (qw$LastChangedRevision: 1528 $)[1];
+our $VERSION = (qw$LastChangedRevision: 1802 $)[1];
 
 
 use strict;
@@ -34,7 +34,6 @@ sub _decode_rdata {			## decode rdata from wire-format octet string
 sub _encode_rdata {			## encode rdata as wire-format octet string
 	my $self = shift;
 
-	return '' unless defined $self->{longitude};
 	pack 'C4N3', @{$self}{qw(version size hp vp latitude longitude altitude)};
 }
 
@@ -42,13 +41,12 @@ sub _encode_rdata {			## encode rdata as wire-format octet string
 sub _format_rdata {			## format rdata portion of RR string.
 	my $self = shift;
 
-	return '' unless defined $self->{longitude};
 	my ( $altitude, @precision ) = map $self->$_() . 'm', qw(altitude size hp vp);
 	my $precision = join ' ', @precision;
 	for ($precision) {
-		s/\s+10m$//;
-		s/\s+10000m$//;
-		s/\s*1m$//;
+		s/^1m 10000m 10m$//;
+		s/ 10000m 10m$//;
+		s/ 10m$//;
 	}
 	my @rdata = ( $self->latitude, '', $self->longitude, '', $altitude, $precision );
 }
