@@ -1,9 +1,9 @@
 package Net::DNS::RR::RRSIG;
 
 #
-# $Id: RRSIG.pm 1729 2019-01-28 09:45:47Z willem $
+# $Id: RRSIG.pm 1779 2020-05-11 09:11:17Z willem $
 #
-our $VERSION = (qw$LastChangedRevision: 1729 $)[1];
+our $VERSION = (qw$LastChangedRevision: 1779 $)[1];
 
 
 use strict;
@@ -29,17 +29,27 @@ use constant DEBUG => 0;
 
 use constant UTIL => defined eval 'use Scalar::Util 1.25; 1;';
 
-# IMPORTANT: Distros MUST NOT create dependencies on Net::DNS::SEC	(strong crypto prohibited in many territories)
-use constant EXISTS => join '', qw(r e q u i r e);		# Defeat static analysers and grep
-use constant DNSSEC => defined( eval join ' ', EXISTS, 'Net::DNS::SEC::Private' );
-use constant ACTIVE => DNSSEC && $INC{'Net/DNS/SEC.pm'};	# Discover how we got here, without loading libcrypto
+# IMPORTANT: Downstream distros MUST NOT create dependencies on Net::DNS::SEC	(strong crypto prohibited in many territories)
+use constant USESEC => defined $INC{'Net/DNS/SEC.pm'};		# Discover how we got here, without exposing any crypto
+								# Discourage static code analysers and casual greppers
+use constant DNSSEC => USESEC && defined eval join '', qw(r e q u i r e), ' Net::DNS', qw(:: SEC :: Private);
 
-my ($RSA) = grep ACTIVE && defined( eval join ' ', EXISTS, $_ ), 'Net::DNS::SEC::RSA';
-my ($DSA) = grep ACTIVE && defined( eval join ' ', EXISTS, $_ ), 'Net::DNS::SEC::DSA';
+my @index;
+if (DNSSEC) {
+	my $key = new Net::DNS::RR( type => 'DNSKEY', key => 'AwEAAQ==' );
+	foreach my $class ( map "Net::DNS::SEC::$_", qw(RSA DSA ECCGOST ECDSA EdDSA) ) {
+		my @algorithms = eval join '', qw(r e q u i r e), " $class; $class->_index";	# 1.14 API
+		@algorithms = grep eval { $key->algorithm($_); $class->verify( '', $key, '' ); 1 }, ( 1 .. 16 )
+				unless scalar(@algorithms);	# Grotesquely inefficient; but need to support older API
+		push @index, map( ( $_ => $class ), @algorithms );
+	}
+}
 
-my ($ECDSA)   = grep ACTIVE && defined( eval join ' ', EXISTS, $_ ), 'Net::DNS::SEC::ECDSA';
-my ($EdDSA)   = grep ACTIVE && defined( eval join ' ', EXISTS, $_ ), 'Net::DNS::SEC::EdDSA';
-my ($ECCGOST) = grep ACTIVE && defined( eval join ' ', EXISTS, $_ ), 'Net::DNS::SEC::ECCGOST';
+my %DNSSEC_verify = @index;
+my %DNSSEC_siggen = @index;
+
+my @deprecated = ( 1, 3, 6, 12 );				# RFC8624
+delete @DNSSEC_siggen{@deprecated};
 
 my @field = qw(typecovered algorithm labels orgttl sigexpiration siginception keytag);
 
@@ -119,7 +129,7 @@ sub _defaults {				## specify RR attribute default values
 	my %algbyval = reverse @algbyname;
 
 	my @algrehash = map /^\d/ ? ($_) x 3 : do { s/[\W_]//g; uc($_) }, @algbyname;
-	my %algbyname = @algrehash;    # work around broken cperl
+	my %algbyname = @algrehash;				# work around broken cperl
 
 	sub _algbyname {
 		my $arg = shift;
@@ -137,30 +147,6 @@ sub _defaults {				## specify RR attribute default values
 }
 
 
-my %DNSSEC_verify = (
-	1  => $RSA,
-	3  => $DSA,
-	5  => $RSA,
-	6  => $DSA,
-	7  => $RSA,
-	8  => $RSA,
-	10 => $RSA,
-	12 => $ECCGOST,
-	13 => $ECDSA,
-	14 => $ECDSA,
-	15 => $EdDSA,
-	16 => $EdDSA,
-	);
-
-my %DNSSEC_sign = (
-	%DNSSEC_verify,
-	1  => 0,			## deprecated ##
-	3  => 0,			## deprecated ##
-	6  => 0,			## deprecated ##
-	12 => 0,			## deprecated ##
-	);
-
-
 sub typecovered {
 	my $self = shift;
 	$self->{typecovered} = typebyname(shift) if scalar @_;
@@ -262,7 +248,7 @@ sub signature { &sig; }
 
 sub create {
 	unless (DNSSEC) {
-		croak 'Net::DNS::SEC support not available';
+		croak qq[No "use Net::DNS::SEC" declaration in application code];
 	} else {
 		my ( $class, $rrsetref, $priv_key, %etc ) = @_;
 
@@ -272,8 +258,7 @@ sub create {
 
 		# All the TTLs need to be the same in the data RRset.
 		my $ttl = $RR->ttl;
-		my @ttl = grep $_->ttl != $ttl, @$rrsetref;
-		croak 'RRs in RRset do not have same TTL' if scalar @ttl;
+		croak 'RRs in RRset do not have same TTL' if grep $_->ttl != $ttl, @$rrsetref;
 
 		my $private = ref($priv_key) ? $priv_key : Net::DNS::SEC::Private->new($priv_key);
 		croak 'unable to parse private key' unless ref($private) eq 'Net::DNS::SEC::Private';
@@ -301,7 +286,8 @@ sub create {
 		$self->{sigexpiration} = $self->{siginception} + $self->{sigval}
 				unless $self->{sigexpiration};
 
-		$self->_CreateSig( $self->_CreateSigData($rrsetref), $private );
+		my $sigdata = $self->_CreateSigData($rrsetref);
+		$self->_CreateSig( $sigdata, $private );
 		return $self;
 	}
 }
@@ -313,11 +299,10 @@ sub verify {
 
 	# $rrsetref must be a reference to an array of RR objects.
 
-	# $keyref is either a key object or a reference to an array
-	# of key objects.
+	# $keyref is either a key object or a reference to an array of key objects.
 
 	unless (DNSSEC) {
-		croak 'Net::DNS::SEC support not available';
+		croak qq[No "use Net::DNS::SEC" declaration in application code];
 	} else {
 		my ( $self, $rrsetref, $keyref ) = @_;
 
@@ -376,7 +361,8 @@ sub verify {
 			return 0;
 		}
 
-		$self->_VerifySig( $self->_CreateSigData($rrsetref), $keyref ) || return 0;
+		my $sigdata = $self->_CreateSigData($rrsetref);
+		$self->_VerifySig( $sigdata, $keyref ) || return 0;
 
 		# time to do some time checking.
 		my $t = time;
@@ -487,9 +473,6 @@ sub _CreateSigData {
 
 		print "_CreateSigData\n" if DEBUG;
 
-		croak 'SIG0 using RRSIG not permitted' unless ref($rrsetref);
-
-		my @field = qw(typecovered algorithm labels orgttl sigexpiration siginception keytag);
 		my $sigdata = pack 'n C2 N3 n a*', @{$self}{@field}, $self->{signame}->canonical;
 		print "\npreamble\t", unpack( 'H*', $sigdata ), "\n" if DEBUG;
 
@@ -497,16 +480,16 @@ sub _CreateSigData {
 		my $limit = $self->{labels};
 		my @label = $owner->_wire;
 		shift @label while scalar @label > $limit;
-		my $wild = bless {label => \@label}, ref($owner);    # DIY to avoid wrecking name cache
+		my $wild   = bless {label => \@label}, ref($owner);    # DIY to avoid wrecking name cache
 		my $suffix = $wild->canonical;
 		unshift @label, chr(42);			# asterisk
 
-		my @RR	  = map bless( {%$_}, ref($_) ), @$rrsetref; # shallow RR clone
-		my $RR	  = $RR[0];
-		my $class = $RR->class;
-		my $type  = $RR->type;
+		my @RR	  = map bless( {%$_}, ref($_) ), @$rrsetref;   # shallow RR clone
+		my $rr	  = $RR[0];
+		my $class = $rr->class;
+		my $type  = $rr->type;
+		my $ttl	  = $self->orgttl;
 
-		my $ttl = $self->orgttl;
 		my %table;
 		foreach my $RR (@RR) {
 			my $ident = $RR->{owner}->canonical;
@@ -551,12 +534,11 @@ sub _CreateSig {
 		my $self = shift;
 
 		my $algorithm = $self->algorithm;
-		my $class     = $DNSSEC_sign{$algorithm};
+		my $class     = $DNSSEC_siggen{$algorithm};
 
 		eval {
 			return $self->sigbin( $class->sign(@_) ) if $class;
-			die qq[algorithm $algorithm not supported\n] if ACTIVE;
-			die qq[No "use Net::DNS::SEC" declaration in application code\n];
+			die "algorithm $algorithm not supported\n";
 		} || croak "${@}signature generation failed";
 	}
 }
@@ -571,8 +553,7 @@ sub _VerifySig {
 
 		my $retval = eval {
 			return $class->verify( @_, $self->sigbin ) if $class;
-			die qq[algorithm $algorithm not supported\n] if ACTIVE;
-			die qq[No "use Net::DNS::SEC" declaration in application code\n];
+			die "algorithm $algorithm not supported\n";
 		};
 
 		unless ($retval) {
@@ -582,7 +563,7 @@ sub _VerifySig {
 		}
 
 		# uncoverable branch true	# bug in Net::DNS::SEC or dependencies
-		croak 'unknown error in $class->verify' unless $retval == 1;
+		croak "unknown error in $class->verify" unless $retval == 1;
 		print "\nalgorithm $algorithm verification successful\n" if DEBUG;
 		return 1;
 	}
@@ -870,11 +851,7 @@ DEALINGS IN THE SOFTWARE.
 =head1 SEE ALSO
 
 L<perl>, L<Net::DNS>, L<Net::DNS::RR>, L<Net::DNS::SEC>,
-RFC4034, RFC6840, RFC3755,
-L<Net::DNS::SEC::DSA>,
-L<Net::DNS::SEC::ECDSA>,
-L<Net::DNS::SEC::EdDSA>,
-L<Net::DNS::SEC::RSA>
+RFC4034, RFC6840, RFC3755
 
 L<Algorithm Numbers|http://www.iana.org/assignments/dns-sec-alg-numbers>
 
