@@ -1,9 +1,9 @@
 package Net::DNS::Resolver::Base;
 
 #
-# $Id: Base.pm 1786 2020-06-15 15:05:47Z willem $
+# $Id: Base.pm 1802 2020-08-31 13:44:20Z willem $
 #
-our $VERSION = (qw$LastChangedRevision: 1786 $)[1];
+our $VERSION = (qw$LastChangedRevision: 1802 $)[1];
 
 
 #
@@ -27,6 +27,7 @@ our $VERSION = (qw$LastChangedRevision: 1786 $)[1];
 
 
 use constant USE_SOCKET_IP => defined eval 'use IO::Socket::IP 0.38; 1';
+require IO::Socket::INET unless USE_SOCKET_IP;
 
 use constant IPv6 => USE_SOCKET_IP;
 
@@ -967,11 +968,15 @@ sub _make_query_packet {
 
 	my ($packet) = @_;
 	if ( ref($packet) ) {
+		my $edns = $packet->edns;			# advertise UDPsize for local stack
+		$edns->size( $self->{udppacketsize} ) unless defined $edns->{size};
+
 		my $header = $packet->header;
 		$header->rd( $self->{recurse} ) if $header->opcode eq 'QUERY';
 
 	} else {
 		$packet = Net::DNS::Packet->new(@_);
+		$packet->edns->size( $self->{udppacketsize} );
 
 		my $header = $packet->header;
 		$header->ad( $self->{adflag} );			# RFC6840, 5.7
@@ -980,8 +985,6 @@ sub _make_query_packet {
 		$header->rd( $self->{recurse} );
 	}
 
-	$packet->edns->size( $self->{udppacketsize} );		# advertise UDPsize for local stack
-
 	if ( $self->{tsig_rr} ) {
 		$packet->sign_tsig( $self->{tsig_rr} ) unless $packet->sigrr;
 	}
